==> static-0.0.2.jar: chia/static/assets.clj <==
(ns chia.static.assets
  (:require [clojure.java.io :as io]
            [hiccup.page :as page]
            [clojure.spec.alpha :as s]
            [clojure.string :as str])
  (:import (java.security MessageDigest)))

;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Asset handling

;; Dynamic vars are used for asset-path options


(def ^:dynamic *content-hashes?*
  "When true, append content-hashes to asset paths."
  false)

(def ^:dynamic *asset-dir*
  "Local directory where assets are written"
  nil)

(def ^:dynamic *asset-host*
  "Remote host where assets are served"
  "")

(defn strip-slash [path]
  (cond-> path
          (str/starts-with? path "/") (subs 1)))

(defn md5
  "Returns md5 hash for string"
  ;; from https://gist.github.com/jizhang/4325757#gistcomment-2196746
  [^String s]
  (let [algorithm (MessageDigest/getInstance "MD5")
        raw (.digest algorithm (.getBytes s))]
    (format "%032x" (BigInteger. 1 raw))))

(defn try-slurp [file]
  (try (slurp file)
       (catch Exception e nil)))

(defn asset-file [path]
  (assert *asset-dir* "*asset-dir* must be set")
  (io/file *asset-dir* (strip-slash path)))

(defn read-asset
  "Returns the contents for an asset"
  [path]
  (-> (asset-file path)
      (try-slurp)))

(defn asset-hash [path]
  (some-> (read-asset path)
          (md5)))

(defn asset-path
  "Asset-path function, for use in generating HTML"
  [path]
  (if-not (str/starts-with? path "/")
    path
    (str
     *asset-host*
     path
     (when *content-hashes?*
       (str "?v=" (asset-hash path))))))

(defn write-asset!
  "Write `content` string to an asset file"
  [path content]
  (-> (asset-file path)
      (spit content))
  (println (str " + " path)))

;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; HTML generation

(defn element [kw-or-hiccup]
  (cond-> kw-or-hiccup
          (keyword? kw-or-hiccup) (vector)))

(defn script [str-or-map]
  [:script (cond (string? str-or-map)
                 str-or-map
                 (map? str-or-map)
                 (update str-or-map :src asset-path)
                 :else str-or-map)])

(defn html-page
  "Return HTML string for title and props"
  [title {:as props
          :keys [page/lang
                 meta/charset
                 head/styles]
          :or {lang "en"
               charset "UTF-8"}}]
  (page/html5 {:lang lang}

              [:head
               [:meta {:http-equiv "Content-Type"
                       :content (str "text/html; charset=" charset)}]

               (when title
                 [:title title])

               (for [href styles]
                 [:link {:rel "stylesheet"
                         :href (asset-path href)}])

               (map element (:head/elements props))
               (map script (:head/scripts props))]

              [:body
               (map element (:body/elements props))
               (map script (:body/scripts props))]))

;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Specs

(s/def :page/lang string?)
(s/def :meta/charset string?)
(s/def :head/styles (s/coll-of string?))
(s/def :body/elements (s/coll-of keyword?))
(s/def :body/scripts (s/coll-of (s/or :eval string?
                                      :include map?)))
(s/def :body/eval (s/coll-of string?))

(s/def ::html-page-props
  (s/keys :opt [:page/lang
                :meta/charaset
                :head/styles
                :body/elements
                :body/scripts
                :body/eval]))

(s/fdef html-page
        :args (s/cat :title string? :props ::html-page-props)
        :ret string?)

==> static-0.0.3.jar: chia/static/assets.clj <==
(ns chia.static.assets
  (:require [clojure.java.io :as io]
            [hiccup.page :as page]
            [clojure.spec.alpha :as s]
            [clojure.string :as str])
  (:import (java.security MessageDigest)))

;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Asset handling

;; Dynamic vars are used for asset-path options


(def ^:dynamic *content-hashes?*
  "When true, append content-hashes to asset paths."
  false)

(def ^:dynamic *asset-dir*
  "Local directory where assets are written"
  nil)

(def ^:dynamic *asset-host*
  "Remote host where assets are served"
  "")

(defn strip-slash [path]
  (cond-> path
          (str/starts-with? path "/") (subs 1)))

(defn md5
  "Returns md5 hash for string"
  ;; from https://gist.github.com/jizhang/4325757#gistcomment-2196746
  [^String s]
  (let [algorithm (MessageDigest/getInstance "MD5")
        raw (.digest algorithm (.getBytes s))]
    (format "%032x" (BigInteger. 1 raw))))

(defn try-slurp [file]
  (try (slurp file)
       (catch Exception e nil)))

(defn asset-file [path]
  (assert *asset-dir* "*asset-dir* must be set")
  (io/file *asset-dir* (strip-slash path)))

(defn read-asset
  "Returns the contents for an asset"
  [path]
  (-> (asset-file path)
      (try-slurp)))

(defn asset-hash [path]
  (some-> (read-asset path)
          (md5)))

(defn asset-path
  "Asset-path function, for use in generating HTML"
  [path]
  (if-not (str/starts-with? path "/")
    path
    (str
     *asset-host*
     path
     (when *content-hashes?*
       (str "?v=" (asset-hash path))))))

(defn write-asset!
  "Write `content` string to an asset file"
  [path content]
  (-> (asset-file path)
      (spit content))
  (println (str " + " path)))

;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; HTML generation

(defn element-tag [kw-or-hiccup]
  (cond-> kw-or-hiccup
          (keyword? kw-or-hiccup) (vector)))

(defn script-tag [str-or-map]
  [:script (cond (string? str-or-map)
                 str-or-map
                 (map? str-or-map)
                 (update str-or-map :src asset-path)
                 :else str-or-map)])

(defn meta-tag [k v]
  [:meta {(if (some #{"Expires"
                      "Pragma"
                      "Cache-Control"
                      "imagetoolbar"
                      "x-dns-prefetch-control"} (name k))
            :http-equiv
            :name) (name k)
          :content v}])

(defn html-page
  "Return HTML string for title and props"
  [title {:as props
          :keys [page/lang
                 meta/charset
                 head/styles
                 head/meta
                 body/elements
                 body/scripts]
          :or {lang "en"
               charset "UTF-8"}}]
  (page/html5 {:lang lang}

              [:head

               (for [[k v] meta]
                 (meta-tag k v))

               [:meta {:http-equiv "Content-Type"
                       :content (str "text/html; charset=" charset)}]

               (when title
                 [:title title])

               (for [href styles]
                 [:link {:rel "stylesheet"
                         :href (asset-path href)}])

               (map element-tag (:head/elements props))
               (map script-tag (:head/scripts props))]

              [:body
               (map element-tag elements)
               (map script-tag scripts)]))

;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Specs

(s/def :page/lang string?)
(s/def :meta/charset string?)
(s/def :head/styles (s/coll-of string?))
(s/def :head/meta (s/map-of keyword?
                            string?))

(s/def ::src string?)
(s/def ::script-props (s/keys
                       :req-un [::src]))
(s/def :body/elements (s/coll-of (s/or :tag keyword?
                                       :element vector?)))
(s/def :body/scripts (s/coll-of (s/or :eval string?
                                      :include map?)))

(s/def ::html-page-props
  (s/keys :opt [:page/lang
                :meta/charaset
                :head/styles
                :body/elements
                :body/scripts
                :body/eval]))

(s/fdef html-page
        :args (s/cat :title string? :props ::html-page-props)
        :ret string?)
